/*  gas liquid solid        */
/* coded by Ikeuchi Mitsuru */
/* ver 0.0.1   2002.05.27   */
/* ver 0.0.2   2002.07.17   */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class gasLiquidSolid extends Applet
  implements AdjustmentListener, Runnable {

/*-------------------------------------- define gloval -----*/

  Scrollbar scr;
  Thread th = null;

  Dimension dim;
  Image imgOff;
  Graphics gOff;

  int sleepTime = 50;
  int atomSizeX = 20; 
  int atomSizeY = 20;  
  int dispR = 10;         /* display molecular radious */
  int paintSizeX = atomSizeX*dispR; 
  int paintSizeY = atomSizeY*dispR;
  int wakuX = paintSizeX + dispR;
  int wakuY = paintSizeY + dispR;

  int Nmt = 36;               /* number of molecules    */
  int NN = Nmt + 1;
  double t = 0.0;              /* time (s)               */
  double dt = 25.0*1.0e-15;    /* time division (s)      */
  double sgm = 3.40e-10;       /* L-J sigma for Ar (m)   */
  double eps = 1.67e-21;       /* L-J epsilon for Ar (J) */
  double mas = 39.95*1.67e-27; /* mass of Ar (kg)        */

  double contTemp = 300.0;

  double xMax = atomSizeX*sgm; /* x-Box size in (m)      */
  double yMax = atomSizeY*sgm; /* y-Box size in (m)      */
  
  double xx[] = new double[NN];   /* i-th x-position */
  double yy[] = new double[NN];   /* i-th y-position */
  double vx[] = new double[NN];   /* i-th x-velocity */
  double vy[] = new double[NN];   /* i-th y-velocity */
  double ffx[] = new double[NN];  /* i-th x-force    */
  double ffy[] = new double[NN];  /* i-th y-force    */
  double fbx[] = new double[NN];  /* i-th x-force before dt */
  double fby[] = new double[NN];  /* i-th y-force before dt */

/*----------------------------- applet control functions -----*/

  public void init() {
    resize(400,250);
    setBackground(Color.white);

    dim = getSize();
    imgOff = createImage(dim.width,dim.height);
    gOff = imgOff.getGraphics();

    scr= new Scrollbar(Scrollbar.HORIZONTAL,300,100,10,700);
    add(new Label("control Temp =")); add(scr);
    scr.addAdjustmentListener(this);

    setInitialPosition();
    repaint();
  }

  public void start() {
    if (th == null) {
      th = new Thread(this);
      th.start();
    }
  }

  public void stop() {
    if (th != null) {
      th.stop();
      th = null;
    }
  }

  public void adjustmentValueChanged(AdjustmentEvent ev){
    if(ev.getSource() == scr) { 
      contTemp = (double)(scr.getValue());
      offPaint();
      repaint();
    }
  }

  public void run() {
    while (th != null) {
      try {
        calcposition();
        offPaint();
        repaint();
        Thread.sleep(sleepTime);
      } catch (InterruptedException e) { }
    }
  }


  public void update(Graphics g) {
    paint(g);
  }

  public void paint(Graphics g) {
    g.drawImage(imgOff,0,0,this);
  }


/* ----------------------------- offPaint -------------------- */

  void offPaint() {
    int i,ix,iy,ir;
    double tmp;

    gOff.setColor(Color.white);
    gOff.fillRect(0,0,dim.width,dim.height);

    tmp = temperature();
    gOff.setColor(Color.black); 
    if (tmp<contTemp) { gOff.setColor(Color.red); };
    gOff.drawRect(30,30,wakuX,wakuY);

    gOff.setColor(Color.blue);
    for (i=0; i<Nmt; i++) {
      ix = (int)(paintSizeX*xx[i]/xMax)+30;
      iy = (int)(paintSizeY*yy[i]/yMax)+30;
      ir = 10;
      gOff.fillOval(ix,iy, ir, ir);
    }

    gOff.setColor(Color.black);
    gOff.drawString("cont T="+contTemp+"K",300,50);
    gOff.setColor(Color.blue);
    gOff.drawString("t="+Math.floor(t*1.0e12)+"(ps)",300,70);
    gOff.drawString("v="+rmsV()+" m/s",300,90);
    gOff.drawString("T="+temperature()+" K",300,110);

  }

/*-------------------------------------  statictics  ----------*/


  double rmsV() {
    int i;
    double s;

    s = 0.0; 
    for (i=0; i<Nmt; i++) {
      s += vx[i]*vx[i] + vy[i]*vy[i];
    }
    return ((double)Math.floor(Math.sqrt(s/(double)Nmt)*100.0)/100.0);
  }

  double temperature() {
    int i;
    double s;

    s = 0.0; 
    for (i=0; i<Nmt; i++) {
      s += vx[i]*vx[i] + vy[i]*vy[i];
    }
    return ((double)Math.floor(mas*s/((double)Nmt*2.0*1.38e-23)*10.0)/10.0 );
  }

/*--------------------------  set initial position   ---------*/

  void setInitialPosition(){
    int i; 
    for (i=0; i<Nmt; i++) {
      xx[i] = sgm*3.0*(double)(i%6)+2.5*sgm;
      yy[i] = sgm*3.0*(double)(i/6)+2.5*sgm;
      vx[i] = 500.0*Math.random()-250.0;
      vy[i] = 500.0*Math.random()-250.0;
      ffx[i] = 0.0;
      ffy[i] = 0.0;
    }
  }

/*-------------------------------  molecules motion   ---------*/

  void calcposition(){
    int i;

    for (i=0; i<20; i++) {
      timeEvolution();
    }
  }

  void timeEvolution(){
    int i; 
    double rr, tmp;

    t = t + dt;

    tmp = temperature();
    rr = Math.sqrt(contTemp/tmp);
    if (rr<0.5) { rr = 0.5; } else if (rr>1.2){ rr=1.2; };

    for (i=0; i<Nmt; i++) {
      xx[i] += vx[i]*dt+0.5*dt*dt*ffx[i]/mas;
      yy[i] += vy[i]*dt+0.5*dt*dt*ffy[i]/mas;
      fbx[i] = ffx[i];
      fby[i] = ffy[i];
    }
    forceCalc();
    for (i=0; i<Nmt; i++) {
      vx[i] += 0.5*dt*(ffx[i]+fbx[i])/mas;
      vy[i] += 0.5*dt*(ffy[i]+fby[i])/mas;
    }

    for (i=0; i<Nmt; i++) {
      if (xx[i] < 0.0) { 
        xx[i] = 0.0; vx[i] = -rr*vx[i]; vy[i] = rr*vy[i]; 
      };
      if (xx[i] > xMax) {
        xx[i] = xMax; vx[i] = -rr*vx[i]; vy[i] = rr*vy[i]; 
      };
      if (yy[i] < 0.0) { 
        yy[i] = 0.0; vx[i] = rr*vx[i]; vy[i] = -rr*vy[i]; 
      };
      if (yy[i] > yMax) {
        yy[i] = yMax; vx[i] = rr*vx[i]; vy[i] = -rr*vy[i];
      };

    }
  }

  void forceCalc() {
    int i,j;
    double rij,f,fxij,fyij;

    for(i=0;i<Nmt;i++) {
      ffx[i] = 0.0;
      ffy[i] = 0.0;
    }

    for(i=0;i<Nmt;i++) {
      for(j=i+1;j<Nmt;j++) {
        rij = Math.sqrt((xx[i]-xx[j])*(xx[i]-xx[j])+(yy[i]-yy[j])*(yy[i]-yy[j]));
        f = force(rij);
        fxij = f*(xx[i]-xx[j])/rij;
        fyij = f*(yy[i]-yy[j])/rij;

        ffx[i] += fxij;
        ffy[i] += fyij;

        ffx[j] += -fxij;
        ffy[j] += -fyij;
      }
    }
  }

  double force(double r) {
    double ri,r3,r6;

    ri = sgm/r;
    r3 = ri*ri*ri;
    r6 = r3*r3;
    return (24.0*eps*r6*(2.0*r6-1.0)/r);
  }

/*----- end of molecules motion -----*/

}  

/*----- end of applet -----*/

